﻿//Copyright (C) Microsoft Corporation.  All rights reserved.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Reflection;
using System.Windows.Forms;

namespace SampleSupport {
 
  public static class ObjectDumper {
    public static void Write(object o) {
      Write(o, 0);
    }

    public static void Write(object o, int expandDepth) {
      Write(o, expandDepth, 10);
    }

    public static void Write(object o, int expandDepth, int maxLoadDepth) {
      if (o != null && o is IEnumerable) {
        // Some linq result collections cannot be iterated twice, so we cache them
        // in an ObjectCollectionCache object
        o = new ObjectCollectionCache(o);
      }

      try {
        if (SampleForm.OutputTreeView != null) {
          TreeViewGenerator gen = TreeViewGenerator.Instance;
          gen.ExpandDepth = expandDepth + 1;
          gen.MaxLoadDepth = 10;

          TreeNode node = gen.GetTreeNode(o);
          SampleForm.OutputTreeView.Nodes.Add(node);

        }
      } catch (Exception ex) {
        MessageBox.Show("EXCEPTION: " + ex.ToString());
      }
    }

  }

  public class ObjectCollectionCache : IEnumerable {
    public readonly Type OriginalType;
    public readonly IList Values;

    public ObjectCollectionCache(object o) {
      OriginalType = o.GetType();
      Values = new List<object>();
      foreach (object o2 in (IEnumerable)o) {
        Values.Add(o2);
      }
    }

    public IEnumerator GetEnumerator() {
      return Values.GetEnumerator();
    }
  }

}
