﻿'Copyright (C) Microsoft Corporation.  All rights reserved.

Imports System.Collections
Imports System.Linq
Imports System.IO
Imports System.Reflection

Namespace SampleSupport

  Public NotInheritable Class ObjectDumper
	Private Sub New()
	End Sub
	Public Shared Sub Write(ByVal o As Object)
	  Write(o, 0)
	End Sub

	Public Shared Sub Write(ByVal o As Object, ByVal expandDepth As Integer)
	  Write(o, expandDepth, 10)
	End Sub

	Public Shared Sub Write(ByVal o As Object, ByVal expandDepth As Integer, ByVal maxLoadDepth As Integer)
	  If o IsNot Nothing AndAlso TypeOf o Is IEnumerable Then
		' Some linq result collections cannot be iterated twice, so we cache them
		' in an ObjectCollectionCache object
		o = New ObjectCollectionCache(o)
	  End If

	  Try
		If SampleForm.OutputTreeView IsNot Nothing Then
		  Dim gen As TreeViewGenerator = TreeViewGenerator.Instance
		  gen.ExpandDepth = expandDepth + 1
		  gen.MaxLoadDepth = 10

		  Dim node As TreeNode = gen.GetTreeNode(o)
		  SampleForm.OutputTreeView.Nodes.Add(node)

		End If
	  Catch ex As Exception
		MessageBox.Show("EXCEPTION: " & ex.ToString())
	  End Try
	End Sub

  End Class

  Public Class ObjectCollectionCache
	  Implements IEnumerable
	Public ReadOnly OriginalType As Type
	Public ReadOnly Values As IList

	Public Sub New(ByVal o As Object)
	  OriginalType = o.GetType()
	  Values = New List(Of Object)()
	  For Each o2 As Object In CType(o, IEnumerable)
		Values.Add(o2)
	  Next o2
	End Sub

	Public Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
	  Return Values.GetEnumerator()
	End Function
  End Class

End Namespace
