﻿'Copyright (C) Microsoft Corporation.  All rights reserved.

Imports System.ComponentModel
Imports System.Linq
Imports System.Text
Imports System.IO
Imports IdeaBlade.EntityModel

Namespace SampleSupport


  Partial Friend Class SampleForm
	  Inherits Form
	Private currentHarness As SampleHarness
	Private currentSample As Sample

	Public Shared Property OutputTreeView() As TreeView

	#Region "Constructor (Some changes)"
	Public Sub New(ByVal title As String, ByVal harnesses As List(Of SampleHarness))

	  InitializeComponent()

	  qsCacheOnlyMenuItem.Tag = QueryStrategy.CacheOnly
	  qsDataSourceOnlyMenuItem.Tag = QueryStrategy.DataSourceOnly
	  qsDataSourceThenCacheMenuItem.Tag = QueryStrategy.DataSourceThenCache
	  qsNormal.Tag = QueryStrategy.Normal

	  QueryStrategy = QueryStrategy.DataSourceOnly

	  ' Added following two lines
	  _harnesses = harnesses
	  Dim managers = _harnesses.OfType(Of DevForceSampleHarness)().Select(Function(x) x.EntityManager).Distinct()
	  If managers.Count() <> 1 Then
		Throw New Exception("Exactly one EntityManager instance is expected for this application")
	  End If
	  _entityManager = managers.First()
	  InitImageList()
	  ConfigureStatusStrip()
	  UpdateMenus()

	  Me.Text = title

	  Dim rootNode As New TreeNode(title)
	  rootNode.Tag = Nothing
	  rootNode.ImageKey = "Help"
	  rootNode.SelectedImageKey = "Help"
	  samplesTreeView.Nodes.Add(rootNode)
	  rootNode.Expand()

	  For Each harness As SampleHarness In harnesses
		Dim harnessNode As New TreeNode(harness.Title)
		harnessNode.Tag = Nothing
		harnessNode.ImageKey = "BookStack"
		harnessNode.SelectedImageKey = "BookStack"
		rootNode.Nodes.Add(harnessNode)

		Dim category As String = ""
		Dim categoryNode As TreeNode = Nothing
		For Each sample As Sample In harness
		  If sample.Category <> category Then
			category = sample.Category

			categoryNode = New TreeNode(category)
			categoryNode.Tag = Nothing
			categoryNode.ImageKey = "BookClosed"
			categoryNode.SelectedImageKey = "BookClosed"
			harnessNode.Nodes.Add(categoryNode)
		  End If

		  Dim node As New TreeNode(sample.ToString())
		  node.Tag = sample
		  node.ImageKey = "Item"
		  node.SelectedImageKey = "Item"
		  categoryNode.Nodes.Add(node)
		Next sample
	  Next harness
	End Sub

	#End Region

	Public Property QueryStrategy() As QueryStrategy
	  Get
		  Return _queryStrategy
	  End Get
	  Set(ByVal value As QueryStrategy)
		_queryStrategy = value
		UpdateQsMenu(_queryStrategy)
	  End Set
	End Property

	Private _queryStrategy As QueryStrategy

	Private Sub samplesTreeView_AfterSelect(ByVal sender As Object, ByVal e As TreeViewEventArgs) Handles samplesTreeView.AfterSelect
	  Dim currentNode As TreeNode = samplesTreeView.SelectedNode
	  currentSample = CType(currentNode.Tag, Sample)
	  If currentSample IsNot Nothing Then
		currentHarness = currentSample.Harness
		runButton.Enabled = True
		descriptionTextBox.Text = currentSample.Description
		codeRichTextBox.Clear()
		codeRichTextBox.Text = currentSample.Code
		outputTextBox.Clear()

		treeViewOutput.Nodes.Clear()
		colorizeCode(codeRichTextBox)
	  Else
		currentHarness = Nothing
		runButton.Enabled = False
		descriptionTextBox.Text = "Select a query from the tree to the left."
		codeRichTextBox.Clear()
	  End If
	End Sub

	Private Sub samplesTreeView_AfterExpand(ByVal sender As Object, ByVal e As TreeViewEventArgs) Handles samplesTreeView.AfterExpand
	  Select Case e.Node.Level
		Case 1, 2
		  e.Node.ImageKey = "BookOpen"
		  e.Node.SelectedImageKey = "BookOpen"
	  End Select
	End Sub

	Private Sub samplesTreeView_BeforeCollapse(ByVal sender As Object, ByVal e As TreeViewCancelEventArgs) Handles samplesTreeView.BeforeCollapse
	  Select Case e.Node.Level
		Case 0
		  e.Cancel = True
	  End Select
	End Sub

	Private Sub samplesTreeView_AfterCollapse(ByVal sender As Object, ByVal e As TreeViewEventArgs) Handles samplesTreeView.AfterCollapse
	  Select Case e.Node.Level
		Case 1
		  e.Node.ImageKey = "BookStack"
		  e.Node.SelectedImageKey = "BookStack"

		Case 2
		  e.Node.ImageKey = "BookClosed"
		  e.Node.SelectedImageKey = "BookClosed"
	  End Select
	End Sub

	Private Sub samplesTreeView_DoubleClick(ByVal sender As Object, ByVal e As EventArgs) Handles samplesTreeView.DoubleClick
	  If Me.currentSample IsNot Nothing Then
		RunCurrentSample()
	  End If
	End Sub

	Private Shared Sub colorizeCode(ByVal rtb As RichTextBox)
	  Dim keywords() As String = {"as", "do", "if", "in", "is", "for", "int", "new", "out", "ref", "try", "base", "bool", "byte", "case", "char", "else", "enum", "goto", "lock", "long", "null", "this", "true", "uint", "void", "break", "catch", "class", "const", "event", "false", "fixed", "float", "sbyte", "short", "throw", "ulong", "using", "where", "while", "yield", "double", "extern", "object", "params", "public", "return", "sealed", "sizeof", "static", "string", "struct", "switch", "typeof", "unsafe", "ushort", "checked", "decimal", "default", "finally", "foreach", "partial", "private", "virtual", "abstract", "continue", "delegate", "explicit", "implicit", "internal", "operator", "override", "readonly", "volatile", "interface", "namespace", "protected", "unchecked", "stackalloc", "from", "in", "where", "select", "join", "equals", "let", "on", "group", "by", "into", "orderby", "ascending", "descending", "var"}
	  Dim text As String = rtb.Text

	  rtb.SelectAll()
	  rtb.SelectionColor = rtb.ForeColor

	  For Each keyword As String In keywords
		Dim keywordPos As Integer = rtb.Find(keyword, RichTextBoxFinds.MatchCase Or RichTextBoxFinds.WholeWord)
		Do While keywordPos <> -1
		  Dim commentPos As Integer = text.LastIndexOf("//", keywordPos, StringComparison.OrdinalIgnoreCase)
		  Dim newLinePos As Integer = text.LastIndexOf(vbLf, keywordPos, StringComparison.OrdinalIgnoreCase)
		  Dim quoteCount As Integer = 0
		  Dim quotePos As Integer = text.IndexOf("""", newLinePos + 1, keywordPos - newLinePos, StringComparison.OrdinalIgnoreCase)
		  Do While quotePos <> -1
			quoteCount += 1
			quotePos = text.IndexOf("""", quotePos + 1, keywordPos - (quotePos + 1), StringComparison.OrdinalIgnoreCase)
		  Loop

		  If newLinePos >= commentPos AndAlso quoteCount Mod 2 = 0 Then
			rtb.SelectionColor = Color.Blue
		  End If

		  keywordPos = rtb.Find(keyword, keywordPos + rtb.SelectionLength, RichTextBoxFinds.MatchCase Or RichTextBoxFinds.WholeWord)
		Loop
	  Next keyword

	  rtb.Select(0, 0)
	End Sub

	Private Sub treeViewOutput_BeforeExpand(ByVal sender As Object, ByVal e As TreeViewCancelEventArgs) Handles treeViewOutput.BeforeExpand
	  If _inBeforeExpand Then
		e.Cancel = True
		Return
	  End If
	  Try
		_inBeforeExpand = True
		TreeViewGenerator.Instance.ExpandTreeNode(e.Node)
	  Finally
		_inBeforeExpand = False
	  End Try

	End Sub
	Private _inBeforeExpand As Boolean = False

	Private Sub runButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles runButton.Click
	  RunCurrentSample()
	End Sub

	Private Sub RunCurrentSample()

	  treeViewOutput.Nodes.Clear()
	  Dim hold As Cursor = Me.Cursor
	  Try
		Me.Cursor = Cursors.WaitCursor
		OutputTreeView = treeViewOutput
		currentSample.InvokeSafe(QueryStrategy)
		tabControl1.SelectTab(0)
	  Catch e As Exception
		outputTextBox.Text = e.Message
		tabControl1.SelectTab(1)
	  Finally
		OutputTreeView = Nothing
		Me.Cursor = hold
	  End Try

	End Sub


	#Region "Added Code"

	#Region "Menu Handlers"

	Private Sub exitToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles exitToolStripMenuItem.Click
	  Me.Close()
	End Sub

	Private Sub devForceTraceViewerToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles devForceTraceViewerToolStripMenuItem.Click
	  LaunchTraceViewer()
	End Sub

	Private Sub stuffCacheToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles stuffCacheToolStripMenuItem.Click
	  LoadCache()
	End Sub

	Private Sub clearCacheToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles clearCacheToolStripMenuItem.Click
	  ClearCache()
	End Sub

	Private Sub saveCacheToDiskToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles saveCacheToDiskToolStripMenuItem.Click
	  SaveEntitySet()
	End Sub

	Private Sub restoreCacheFromDiskToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles restoreCacheFromDiskToolStripMenuItem.Click
	  RestoreEntitySet()
	End Sub

	Private Sub qsMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles qsCacheOnlyMenuItem.Click, qsDataSourceOnlyMenuItem.Click, qsDataSourceThenCacheMenuItem.Click, qsNormal.Click
	  Dim menuItem = TryCast(sender, ToolStripMenuItem)
	  Dim qs = TryCast(menuItem.Tag, QueryStrategy)
	  QueryStrategy = qs
	End Sub

	#End Region

	#Region "Cache Operations"

	Private Sub LoadCache()
	  Me._toolStripStatusLabel.Text = "Loading cache..."
	  Application.DoEvents()
	  Dim origCursor = Me.Cursor
	  Try
		Me.Cursor = Cursors.WaitCursor
		For Each harness In _harnesses.OfType(Of DevForceSampleHarness)()
		  harness.LoadCache()
		Next harness
	  Finally
		Me.Cursor = origCursor
	  End Try

	  ' Note that there are other types (e.g., Employees) referenced in some of the queries
	  ' that we're not loading. Because of that, those queries won't work disconnected.
	  UpdateMenus()
	End Sub

	Private Sub ClearCache()
	  _entityManager.Clear()
	End Sub


	Private Sub SaveEntitySet()
	  Try
		_entityManager.CacheStateManager.SaveCacheState(_entitySetPathName)
		MessageBox.Show("Contents of cache save to local disk file " & _entitySetPathName)
	  Catch e1 As Exception
		MessageBox.Show("Unable to save contents of cache to local disk file " & _entitySetPathName)
	  End Try
	End Sub

	Private Sub RestoreEntitySet()

	  Try
		_entityManager.CacheStateManager.RestoreCacheState(_entitySetPathName, RestoreStrategy.Normal)
		MessageBox.Show("Contents of cache restored from local disk file " & _entitySetPathName)
	  Catch e1 As Exception
		MessageBox.Show("Unable to restore data from local disk file " & _entitySetPathName)
	  End Try
	End Sub


	#End Region

	#Region "UI state maintenance and utility"

	Private Sub InitImageList()
      Me.imageListConnection.Images.Add("Connected", SampleQueries.Resources01.connected)
	  Me.imageListConnection.Images.Add("Disconnected", SampleQueries.Resources01.disconnected)
	End Sub

	Private Sub UpdateMenus()

	  UpdateQsMenu(QueryStrategy)

	  ' Always available items:
	  Me.clearCacheToolStripMenuItem.Enabled = True
	  Me.saveCacheToDiskToolStripMenuItem.Enabled = True
	  Me.restoreCacheFromDiskToolStripMenuItem.Enabled = True

	End Sub

	Private Sub UpdateQsMenu(ByVal queryStrategy As QueryStrategy)
	  For Each item In Me.qsCacheOnlyMenuItem.Owner.Items
		Dim mi = TryCast(item, ToolStripMenuItem)
		mi.Checked = mi.Tag.Equals(queryStrategy)
		If mi.Checked Then
		  _toolStripStatusLabel.Text = mi.Text
		End If
	  Next item
      'Dim connected As Boolean = Me.QueryStrategy IsNot Me.QueryStrategy.CacheOnly
      Dim connected As Boolean
      If Me.QueryStrategy IsNot queryStrategy.CacheOnly Then
        connected = False
      Else
        connected = True
      End If
      Me.stuffCacheToolStripMenuItem.Enabled = connected
      If connected Then
        Me._toolStripStatusLabel.Image = imageListConnection.Images("Connected")
      Else
        Me._toolStripStatusLabel.Image = imageListConnection.Images("Disconnected")
      End If
      ' insure that treeView is never showing data retrieved for an earlier query strategy.
      outputTextBox.Clear()
      treeViewOutput.Nodes.Clear()
    End Sub


	Private Sub ConfigureStatusStrip()
	  Me.statusStrip1.Items.AddRange(New ToolStripItem() { _toolStripStatusLabel})
	End Sub

	Private Shared Sub LaunchTraceViewer()
      Dim tv As New IdeaBlade.DevTools.WPFTraceViewer.WPFTraceViewer()
	  tv.Show()
	End Sub

	#End Region

	#Region "Private Fields"
	Private _harnesses As List(Of SampleHarness)
	Private _entityManager As EntityManager
	Private _colorConnectedNo As Color = Color.Red
	Private _colorConnectedYes As Color = Color.Green
	Private _entitySetPathName As String = "C:\EntitySetCache.bin"
	Private _toolStripStatusLabel As New ToolStripStatusLabel()
	#End Region


	#End Region


  End Class

  Friend MustInherit Class DevForceSampleHarness
	  Inherits SampleHarness

	Public Overrides Sub InitSample(ByVal initializationData As Object)
	  Dim queryStrategy = CType(initializationData, QueryStrategy)
	  EntityManager.DefaultQueryStrategy = queryStrategy
	  If queryStrategy Is QueryStrategy.CacheOnly Then
		EntityManager.Disconnect()
	  Else
		EntityManager.Connect()
	  End If
	End Sub

	Public MustOverride ReadOnly Property EntityManager() As EntityManager

	Public Overridable Sub LoadCache()
	End Sub

  End Class

End Namespace